#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
模拟通道模块

定义模拟通道类，继承自Analog和DmfChannel，用于表示电力系统中的模拟量测量通道。
模拟量通道用于采集电压、电流等连续变化的物理量。
"""
from typing import Any, Optional
from xml.etree.ElementTree import Element

import numpy as np
from pydantic import Field, field_serializer

from comtrade_io.cfg import Analog
from comtrade_io.dmf.dmf_channel import DmfChannel
from comtrade_io.type import (AnalogChannelFlag, AnalogChannelType, Phase, TranSide, Unit)
from comtrade_io.utils import parse_float, parse_int
from comtrade_io.utils.channel_recognizer import recognize_channel


class AnalogChannel(Analog, DmfChannel):
    """
    模拟通道类
    
    表示电力系统中的模拟量测量通道，用于采集电压、电流等连续变化的物理量。
    继承自Analog和DmfChannel，包含了通道的标幺值、一次侧/二次侧量程等属性。
    
    属性:
        p_min: 通道一次侧量程的最小值，仅对直流类型有效
        p_max: 通道一次侧量程的最大值，仅对直流类型有效
        s_min: 通道二次侧量程的最小值，仅对直流类型有效
        s_max: 通道二次侧量程的最大值，仅对直流类型有效
        freq: 模拟量频率，通常为50Hz或60Hz
        au: 模拟量标幺系数，用于将实际值转换为标幺值
        bu: 模拟量标幺偏移量
        unit_multiplier: 模拟量增益系数，用于单位换算
        data: 通道数据，一维数组
    """
    p_min: float = Field(default=0.0, description="通道一次侧量程的最小值，仅对直流类型有效")
    p_max: float = Field(default=0.0, description="通道一次侧量程的最大值，仅对直流类型有效")
    s_min: float = Field(default=0.0, description="通道二次侧量程的最小值，仅对直流类型有效")
    s_max: float = Field(default=0.0, description="通道二次侧量程的最大值，仅对直流类型有效")
    freq: float = Field(default=50.0, description="模拟量频率")
    au: float = Field(default=1.0, description="模拟量标幺")
    bu: float = Field(default=0.0, description="模拟量标幺")
    unit_multiplier: str = Field(default="", description="模拟量增益系数")
    data: Optional[Any] = Field(default=None, description="通道数据，一维数组")

    @field_serializer('data')
    def serialize_data(self, data: Any) -> Optional[list]:
        if data is None:
            return None
        if isinstance(data, np.ndarray):
            return data.tolist()
        return data

    def __str__(self):
        """
        返回模拟通道的XML字符串表示形式
        
        返回:
            格式化的XML字符串，表示模拟通道的所有属性
        """
        attrs = [
            f'idx_cfg="{self.index}"',
            f'idx_org="{self.idx_org}"',
            f'type="{self.type.value}"',
            f'flag="{self.flag.value}"',
            f'freq="{self.freq}"',
            f'au="{self.au}"',
            f'bu="{self.bu}"',
            f'sIUnit="{self.unit.value}"',
            f'multiplier="{self.unit_multiplier}"',
            f'primary="{self.primary}"',
            f'secondary="{self.secondary}"',
            f'ps="{self.tran_side.value}"',
            f'ph="{self.phase.value}"',
        ]
        return f"\t<scl:AnalogChannel {''.join(attrs)} />"

    @classmethod
    def from_xml(cls, element: Element, analog: Optional[Analog] = None) -> "AnalogChannel":
        """从XML元素创建AnalogChannel实例

        参数:
            element (Element): XML元素
            ns (dict, optional): 命名空间映射
            analog (Analog, optional): 用于对比和更新的Analog实例

        返回:
            AnalogChannel: AnalogChannel实例
        """
        _type = AnalogChannelType.from_value(element.get('type', ''))
        _flag = AnalogChannelFlag.from_value(element.get('flag', ''))
        if _type != _flag.type:
            _type = _flag.type
        _unit_str = element.get('sIUnit', '')
        _phase_str = element.get('ph', '')
        _tran_side_str = element.get('ps', '')
        channel = cls(
            index=parse_int(element.get('idx_cfg', 1)),
            idx_org=parse_int(element.get('idx_org', 1)),
            type=_type,
            flag=_flag,
            p_min=parse_float(element.get('p_min', 0.0)),
            p_max=parse_float(element.get('p_max', 0.0)),
            s_min=parse_float(element.get('s_min', 0.0)),
            s_max=parse_float(element.get('s_max', 0.0)),
            freq=parse_float(element.get('freq', 50.0)),
            au=parse_float(element.get('au', 1.0)),
            bu=parse_float(element.get('bu', 0.0)),
            unit=Unit.from_value(_unit_str),
            unit_multiplier=element.get('multiplier', ""),
            phase=Phase.from_value(_phase_str),
            primary=parse_float(element.get('primary', 1.0)),
            secondary=parse_float(element.get('secondary', 1.0)),
            tran_side=TranSide.from_value(_tran_side_str)
        )

        if analog is not None:
            analog_dict = analog.model_dump()
            for key, value in analog_dict.items():
                if hasattr(channel, key):
                    current_value = getattr(channel, key)
                    if current_value != value:
                        setattr(channel, key, value)

        return channel

    @classmethod
    def from_analog(cls, analog: Analog) -> "AnalogChannel":
        """从Analog实例创建AnalogChannel实例

        该方法会自动根据通道名称识别通道类型和标志:
        - AnalogChannelType: 交流(A)、直流(D)、其他(O)
        - AnalogChannelFlag: 电压(ACV)、电流(ACC)、高频(HF)、频率(FQ)等

        参数:
            analog (Analog): Analog实例

        返回:
            AnalogChannel: AnalogChannel实例
        """
        # 将Analog实例转换为字典
        analog_dict = analog.model_dump()

        # 设置必需的 idx_org 字段，默认与 index 相同
        analog_dict['idx_org'] = analog_dict.get('index', 1)

        # 根据通道名称自动识别通道类型和标志
        if analog.name:
            channel_type, channel_flag = recognize_channel(analog.name)
            analog_dict['type'] = channel_type
            analog_dict['flag'] = channel_flag

        # 设置默认频率（如果未指定）
        if 'freq' not in analog_dict or analog_dict.get('freq', 50.0) == 0.0:
            analog_dict['freq'] = 50.0

        # 设置默认标幺系数
        if 'au' not in analog_dict:
            analog_dict['au'] = 1.0
        if 'bu' not in analog_dict:
            analog_dict['bu'] = 0.0

        # 设置默认倍率
        if 'unit_multiplier' not in analog_dict:
            analog_dict['unit_multiplier'] = ""

        # 使用字典创建AnalogChannel实例
        return cls(**analog_dict)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
DMF通道模块

定义DMF数据模型中的通道基类，包括模拟通道和开关量通道的通用属性。
"""

from pydantic import BaseModel, Field

from comtrade_io.type import (AnalogChannelFlag, AnalogChannelType, DigitalChannelFlag, DigitalChannelType)


class DmfChannel(BaseModel):
    """
    DMF通道基类
    
    定义模拟量和开关量通道的通用属性，包括端子排号、通道类型和通道标志。
    
    属性:
        idx_org: 端子排号，表示通道在端子排上的原始位置
        type: 通道类型，可以是模拟通道类型或数字通道类型
        flag: 通道标志，用于标识通道的具体用途
    """
    idx_org: int = Field(ge=0, description="端子排号")
    type: AnalogChannelType | DigitalChannelType | None = Field(default=None, description="通道类型")
    flag: AnalogChannelFlag | DigitalChannelFlag | None = Field(default=None, description="通道标识")

    def __str__(self) -> str:
        """
        返回通道的字符串表示形式
        
        返回:
            格式化的字符串，包含通道的主要属性
        """
        attrs = [f"idx_org={self.idx_org}"]
        if self.type:
            attrs.append(f"type={self.type.value}")
        if self.flag:
            attrs.append(f"flag={self.flag.value}")
        return f"DmfChannel({', '.join(attrs)})"



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import json
from typing import Optional

from pydantic import Field

from comtrade_io.base import IndexBaseModel
from comtrade_io.type.phase import Phase
from comtrade_io.utils.str_split import str_split


class CfgChannelBaseModel(IndexBaseModel):
    """
    Cfg通道基类

    参数:
        index(int): 通道索引（模拟通道是An，数字通道是Dn，统一用index表示）
        name(str): 通道标识（ch_id）
        phase(Phase): 通道相别标识（ph）
        equip(str): 被监视的电路元件（ccbm）
    返回:
        Channel对象
    """
    name: Optional[str] = Field(default=None, description="通道标识")
    phase: Optional[Phase] = Field(default=Phase.NONE, description="通道相别标识")
    equip: Optional[str] = Field(default='', description="被监视的电路元件")

    def __str__(self) -> str:
        """序列化为逗号分隔的字符串

        将通道基类对象转换为COMTRADE配置文件格式的字符串。

        Returns:
            str: 逗号分隔的字符串，格式为 "index,name,phase,equip"
        """
        phase_value = self.phase.value if self.phase else ""
        return f"{self.index},{self.name},{phase_value},{self.equip}"

    @classmethod
    def from_dict(cls, data: dict) -> 'CfgChannelBaseModel':
        """从字典反序列化"""
        name = data.get("name", None)
        if not name:
            raise ValueError(f"传入的参数错误,缺少必填字段name")
        phase = data.get("phase", "")
        if isinstance(phase, str):
            data['phase'] = Phase.from_value(phase)
        return cls(**data)

    @classmethod
    def from_json(cls, json_str: str) -> 'CfgChannelBaseModel':
        """从JSON字符串反序列化"""
        data = json.loads(json_str)
        return cls.from_dict(data)

    @classmethod
    def from_str(cls, _str: str) -> 'CfgChannelBaseModel':
        """从逗号分隔的字符串反序列化通道对象

        将配置文件中的通道信息字符串解析为CfgChannelBaseModel对象。
        字符串格式为: "index,name,phase,equip"

        参数:
            _str: 逗号分隔的通道字符串

        Returns:
            CfgChannelBaseModel: 解析后的通道对象

        异常:
            ValueError: 当字符串格式不正确或缺少必填字段时抛出
        """
        str_arr = str_split(_str)
        if (arr_len := len(str_arr)) < 2:
            raise ValueError(f"字符串分割后数组为[{str_arr}],长度不足")
        channel = cls(index=int(str_arr[0]), name=str_arr[1])
        if arr_len >= 3:
            channel.phase = Phase.from_value(str_arr[2], Phase.NONE)
        if arr_len >= 4:
            channel.equip = str_arr[3]
        return channel



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from comtrade_io.type.base_enum import BaseEnum


class Phase(BaseEnum):
    PHASE_A = ("A", "A相")
    PHASE_B = ("B", "B相")
    PHASE_C = ("C", "C相")
    PHASE_N = ("N", "N相")
    NONE = ("", "无")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""边界检查工具模块"""
from typing import Union


def check_index_range(
        index: int,
        min_value: int,
        max_value: int,
        raise_error: bool = True
) -> bool:
    """
    检查索引是否在有效范围内

    参数:
        index: 要检查的索引值
        min_value: 最小允许值（包含）
        max_value: 最大允许值（包含）
        raise_error: 是否在超出范围时抛出异常

    返回:
        bool: 索引是否在范围内
    """
    if min_value <= index <= max_value:
        return True
    if raise_error:
        raise IndexError(f"索引值应在[{min_value}, {max_value}]之间，输入的: {index}超出范围")
    return False


def check_positive(value: Union[int, float], allow_zero: bool = False) -> bool:
    """
    检查值是否为正数

    参数:
        value: 要检查的值
        allow_zero: 是否允许零值

    返回:
        bool: 值是否有效
    """
    if allow_zero:
        return value >= 0
    return value > 0


def check_not_empty(value: str, field_name: str = "字段") -> bool:
    """
    检查字符串是否非空

    参数:
        value: 要检查的字符串
        field_name: 字段名称（用于错误消息）

    返回:
        bool: 字符串是否非空
    """
    if not value or not value.strip():
        raise ValueError(f"{field_name}不能为空")
    return True


def clamp(value: Union[int, float], min_value: Union[int, float], max_value: Union[int, float]) -> Union[int, float]:
    """
    将值限制在指定范围内

    参数:
        value: 要限制的值
        min_value: 最小值
        max_value: 最大值

    返回:
        Union[int, float]: 限制后的值
    """
    return max(min_value, min(value, max_value))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
文件压缩工具
"""
from __future__ import annotations

import zipfile
from datetime import datetime
from pathlib import Path
from typing import List, Optional, Union

from comtrade_io.utils.logging import get_logger

logger = get_logger(__name__)


def generate_filename_with_timestamp(suffix: str = ".zip", fmt: str = "%Y%m%d_%H%M%S_%f") -> str:
    """
    以当前时间生成文件名

    Args:
        suffix: 文件后缀（如 ".zip"）
        fmt: 时间格式，默认为 "%Y%m%d_%H%M%S_%f"
            例如: "20260224_150223_231000"

    Returns:
        基于当前时间生成的文件名
    """
    timestamp = datetime.now().strftime(fmt)
    # 去掉微秒部分的后三位（保留前三位）
    if "%f" in fmt:
        parts = timestamp.split("_")
        if len(parts) >= 2:
            microseconds_part = parts[-1]
            if len(microseconds_part) > 3:
                parts[-1] = microseconds_part[:3]
                timestamp = "_".join(parts)
    return f"{timestamp}{suffix}"


class FileCompressor:
    """文件压缩工具类"""

    @staticmethod
    def compress_files(
        files: List[Union[str, Path]],
        output_dir: Union[str, Path],
        zip_name: Optional[str] = None,
        preserve_structure: bool = True,
    ) -> Path:
        """
        压缩文件列表到 ZIP 文件

        Args:
            files: 要压缩的文件路径列表（Path 对象或字符串）
            output_dir: 压缩文件输出目录
            zip_name: 压缩文件名（如 "archive.zip"），如果不传入则使用当前时间生成
            preserve_structure: 是否保留原始路径结构，默认保留。
                - True: 保留文件相对于其公共父目录的相对路径
                - False: 所有文件直接放在压缩包根目录（扁平化）

        Returns:
            压缩文件的完整路径

        Note:
            - 路径不存在或不可读的文件将被跳过
            - 会记录跳过文件的警告信息
        """
        output_dir = Path(output_dir)
        output_dir.mkdir(parents=True, exist_ok=True)

        # 如果未传入 zip_name，使用时间戳生成
        if zip_name is None:
            zip_name = generate_filename_with_timestamp(suffix=".zip", fmt="%Y%m%d_%H%M%S_%f")

        zip_path = output_dir / zip_name

        files_to_compress = []
        for file in files:
            file_path = Path(file)
            if not file_path.exists():
                logger.warning(f"文件不存在，跳过: {file_path}")
                continue
            if not file_path.is_file():
                logger.warning(f"不是文件，跳过: {file_path}")
                continue
            if not FileCompressor._is_readable(file_path):
                logger.warning(f"文件不可读，跳过: {file_path}")
                continue
            files_to_compress.append(file_path)

        if not files_to_compress:
            logger.warning("没有可压缩的文件")
            # 创建空的 zip 文件
            with zipfile.ZipFile(zip_path, "w", zipfile.ZIP_DEFLATED) as zipf:
                pass
            return zip_path

        # 计算公共父目录（仅在保留路径结构时需要）
        common_parent = None
        if preserve_structure and len(files_to_compress) > 1:
            try:
                common_parent = Path(
                    str(files_to_compress[0].parent)
                ).resolve()
                for file_path in files_to_compress[1:]:
                    current = Path(str(file_path.parent)).resolve()
                    # 找到公共父目录
                    while not str(current).startswith(str(common_parent)):
                        common_parent = common_parent.parent
            except Exception:
                # 如果无法计算公共父目录，使用第一个文件的目录
                common_parent = files_to_compress[0].parent

        try:
            with zipfile.ZipFile(zip_path, "w", zipfile.ZIP_DEFLATED) as zipf:
                for file_path in files_to_compress:
                    # 确定压缩包中的路径
                    if preserve_structure:
                        if common_parent:
                            try:
                                arcname = file_path.relative_to(common_parent)
                            except ValueError:
                                # 如果文件不在公共父目录下，只用文件名
                                arcname = file_path.name
                        else:
                            arcname = file_path.name
                    else:
                        # 扁平化结构，只保留文件名
                        arcname = file_path.name

                    zipf.write(file_path, str(arcname))
                    logger.debug(f"已添加到压缩包: {arcname}")

            logger.info(f"压缩完成: {zip_path} (共 {len(files_to_compress)} 个文件)")
            return zip_path

        except Exception as e:
            logger.error(f"压缩文件时出错: {e}")
            raise

    @staticmethod
    def _is_readable(path: Path) -> bool:
        """检查文件是否可读"""
        try:
            return path.exists() and path.is_file() and path.stat().st_size >= 0
        except Exception:
            return False


def compress_files(
    files: List[Union[str, Path]],
    output_dir: Union[str, Path],
    zip_name: Optional[str] = None,
    preserve_structure: bool = True,
) -> Path:
    """
    压缩文件列表到 ZIP 文件（函数接口）

    Args:
        files: 要压缩的文件路径列表
        output_dir: 压缩文件输出目录
        zip_name: 压缩文件名（如 "archive.zip"），如果不传入则使用当前时间生成
        preserve_structure: 是否保留路径结构

    Returns:
        压缩文件的完整路径
    """
    return FileCompressor.compress_files(files, output_dir, zip_name, preserve_structure)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
母线模型模块

定义母线类，用于表示电力系统中的母线模型。
母线是电力系统中的重要节点，用于汇集和分配电能。
"""
from xml.etree.ElementTree import Element

from pydantic import Field

from comtrade_io.dmf.branch import ACVBranch
from comtrade_io.dmf.dmf_base_model import DmfBaseModelModel
from comtrade_io.type import TvInstallSite
from comtrade_io.utils import parse_float, parse_int


class Bus(DmfBaseModelModel):
    """
    母线类
    
    表示电力系统中的母线模型，是电力系统网络中的关键节点。
    母线用于汇集和分配电能，连接线路、变压器等设备。
    
    属性:
        v_rtg: 一次额定电压，单位通常为kV
        v_rtg_snd: 二次额定电压（用于保护装置的电压输入），单位通常为V
        v_rtg_snd_pos: TV安装位置，标识电压互感器的安装位置
        voltage: 电压通道，包含该母线的电压通道索引信息
        anas: 模拟通道列表，继承自基类
        stas: 开关量通道列表，继承自基类
    """
    v_rtg: float = Field(default=0.0, description="一次额定电压")
    v_rtg_snd: float = Field(default=100.0, description="二次额定电压")
    v_rtg_snd_pos: TvInstallSite = Field(default=TvInstallSite.BUS, description="TV安装位置")
    voltage: ACVBranch = Field(default=ACVBranch(), description="电压通道")

    def __str__(self):
        """
        返回母线的XML字符串表示形式
        
        返回:
            格式化的XML字符串，包含母线及其所有子元素的完整表示
        """
        attrs = [
            f'idx"={self.index}"',
            f'bus_name="{self.name}"',
            f'srcRef="{self.reference}"',
            f'VRtg="{self.v_rtg}"',
            f'VRtgSnd="{self.v_rtg_snd}"',
            f'VRtgSnd_Pos="{self.v_rtg_snd_pos}"',
            f'bus_uuid="{self.uuid}"'
        ]
        attrs = [attr for attr in attrs if attr is not None]
        xml = f"\t<scl:Bus {' '.join(attrs)}/>"
        xml += "\n\t\t" + str(self.voltage)
        xml += self.get_ana_chn_xml()
        xml += self.get_sta_chn_xml()
        xml += "\n\t</scl:Bus>"
        return xml

    def info(self):
        parts = [
            f'【母线信息】',
            f'母线名称：{self.name},内部编号索引：{self.index}',
            f'一次额定电压：{self.v_rtg}kV,二次额定电压：{self.v_rtg_snd}V',
            f'TV安装位置：{self.v_rtg_snd_pos.description}',
            f'【电压通道情况】'
        ]
        for vp in (self.voltage.ua, self.voltage.ub, self.voltage.uc, self.voltage.ul, self.voltage.un):
            if not vp:
                continue
            chn = f'通道索引号：{vp.index}，通道名称：{vp.name},{vp.phase.value}相{vp.type.description}{vp.flag.description}'
            parts.append(chn)
        return "\n".join(parts)

    @classmethod
    def from_xml(cls, element: Element, ns: dict, analog_channels: dict = None, status_channels: dict = None) -> 'Bus':
        """
        从XML元素中解析母线模型

        参数:
            element: XML元素
            ns: 命名空间映射
            analog_channels: 模拟通道字典（可选），用于解析ACVBranch中的通道对象
            status_channels: 开关量通道字典（可选），用于解析开关量通道对象
            
        返回:
            Bus: 母线实例
        """
        idx = parse_int(element.get('idx', 1))
        bus_name = element.get('bus_name', "")
        src_ref = element.get('srcRef', "")
        v_rtg = parse_float(element.get('VRtg', 0.0))
        v_rtg_snd = parse_float(element.get('VRtgSnd', 100.0))
        v_rtg_snd_pos_str = element.get('VRtgSnd_Pos', "")
        v_rtg_snd_pos = TvInstallSite.from_value(v_rtg_snd_pos_str, default=TvInstallSite.BUS)
        bus_uuid = element.get('bus_uuid', "")
        bus = cls(index=idx, name=bus_name, reference=src_ref, v_rtg=v_rtg, v_rtg_snd=v_rtg_snd,
                  v_rtg_snd_pos=v_rtg_snd_pos, uuid=bus_uuid)

        # 查找 ACVChn 元素（支持带/不带命名空间）
        acv_chn_elem = element.find('scl:ACVChn', ns) if 'scl' in ns else None
        if acv_chn_elem is None:
            acv_chn_elem = element.find('ACVChn')
        if acv_chn_elem is not None:
            bus.voltage = ACVBranch.from_xml(acv_chn_elem, ns, analog_channels=analog_channels)

        # 解析通道（支持带/不带命名空间）
        if 'scl' in ns:
            bus.anas = cls.parse_ans_from_xml(element, ns, analog_channels=analog_channels, use_scl_prefix=True)
            bus.stas = cls.parse_sts_from_xml(element, ns, status_channels=status_channels, use_scl_prefix=True)
        else:
            bus.anas = cls.parse_ans_from_xml(element, ns, analog_channels=analog_channels, use_scl_prefix=False)
            bus.stas = cls.parse_sts_from_xml(element, ns, status_channels=status_channels, use_scl_prefix=False)

        return bus



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
文件操作工具
"""
from __future__ import annotations

import os
from pathlib import Path
from typing import List, Union


class FileOperations:
    """文件操作工具类"""

    @staticmethod
    def find_files(
        directory: Union[str, Path],
        extensions: Union[str, List[str]] = None,
        recursive: bool = False,
        check_readable: bool = True,
    ) -> List[Path]:
        """
        查找指定目录下的文件

        Args:
            directory: 目录路径
            extensions: 后缀名列表，例如 [".cfg", ".dat"]，支持单个字符串或列表，不包含 "." 也可以
            recursive: 是否遍历子目录，默认不遍历
            check_readable: 是否检查文件可读权限，默认检查

        Returns:
            存在且可读的文件路径列表
        """
        dir_path = Path(directory)
        if not dir_path.exists():
            return []
        if not dir_path.is_dir():
            return []

        # 处理扩展名格式
        if extensions is None:
            extensions = []
        elif isinstance(extensions, str):
            extensions = [extensions]
        extensions = [ext if ext.startswith(".") else f".{ext}" for ext in extensions]

        # 选择遍历方法
        paths = dir_path.rglob("*") if recursive else dir_path.glob("*")

        # 过滤文件并检查权限
        result = []
        for path in paths:
            if not path.is_file():
                continue
            if extensions and path.suffix not in extensions:
                continue
            if check_readable and not FileOperations._is_readable(path):
                continue
            result.append(path)

        return result

    @staticmethod
    def find_sibling_files(
        file_path: Union[str, Path],
        extensions: Union[str, List[str]] = None,
        check_readable: bool = True,
    ) -> List[Path]:
        """
        查找同目录下指定后缀名的文件

        Args:
            file_path: 参考文件路径
            extensions: 后缀名列表，例如 [".cfg", ".dat"]，支持单个字符串或列表，不包含 "." 也可以
            check_readable: 是否检查文件可读权限，默认检查

        Returns:
            存在且可读的同目录文件路径列表
        """
        path = Path(file_path)
        if not path.exists():
            return []
        if not path.is_file():
            return []

        # 获取同目录所有文件
        return FileOperations.find_files(
            directory=path.parent,
            extensions=extensions,
            recursive=False,
            check_readable=check_readable,
        )

    @staticmethod
    def _is_readable(path: Path) -> bool:
        """检查文件是否可读"""
        try:
            return os.access(str(path), os.R_OK)
        except Exception:
            return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from pydantic import Field

from comtrade_io.cfg.cfg_channel_model import CfgChannelBaseModel
from comtrade_io.type import TranSide, Unit
from comtrade_io.utils import get_logger, parse_float

logging = get_logger()


def amend_channel_name_error(_channel_arr: list) -> list:
    """
    cfg文件通道信息依赖英文逗号进行分割数据
    通道名称依赖人工录入，在部分录波设备中人员可能错误录入非法"，"
    本函数通过检测第五位索引增益系数是否是数字或第四位索引是否为单位来合并名称中包含的逗号
    :param _channel_arr: 分割后的通道信息列表
    :return: 合并后的通道信息列表
    """
    for i in range(5, len(_channel_arr)):
        try:
            parse_float(_channel_arr[i])
            al_new = [_channel_arr[0], '_'.join(_channel_arr[2:i - 3])]
            al_new.extend(_channel_arr[i - 3:])
            return al_new
        except ValueError:
            continue
    return []


class Analog(CfgChannelBaseModel):
    """
    cfg文件中模拟量通道类

    参数:
        index(int): 通道索引（模拟通道是An，数字通道是Dn，统一用index表示）
        name(str): 通道标识（ch_id）
        phase(Phase): 通道相别标识（ph）
        equip(str): 被监视的电路元件（ccbm）
        unit(Unit): 通道单位
        multiplier(float): 通道增益系数(实数,可使用标准浮点标记法)
        offset(float): 通道偏移量
        delay(float): 通道时滞（μs）
        min_value(float): 数值最小值
        max_value(float): 数值最大值
        primary(float): 互感器一次系数
        secondary(float): 互感器二次系数
        tran_side(TranSide): 转换标识(P/S)
    """
    unit: Unit = Field(default=Unit.NONE, description="通道单位")
    multiplier: float = Field(default=1.0, ge=0.0, description="通道增益系数(实数,可使用标准浮点标记法)")
    offset: float = Field(default=0.0, description="通道偏移量")
    delay: float = Field(default=0.0, description="通道时滞（μs）")
    min_value: float = Field(default=0.0, description="数值最小值")
    max_value: float = Field(default=0.0, description="数值最大值")
    primary: float = Field(default=1.0, description="互感器一次系数")
    secondary: float = Field(default=1.0, description="互感器二次系数")
    tran_side: TranSide = Field(default=TranSide.S, description="转换标识(P/S)")

    def __str__(self):
        """
        返回对象的字符串表示形式

        该方法扩展了父类的__str__方法，在其基础上添加了当前对象的特定属性信息，
        包括单位值、添加标志、偏移量、延迟、最小值、最大值、主次标识和传输侧等信息。

        Returns:
            str: 包含父类字符串表示和当前对象所有属性信息的完整字符串
        """
        return (
                super().__str__()
                + f",{self.unit.value},{self.multiplier},{self.offset},{self.delay}"
                + f",{self.min_value},{self.max_value},{self.primary},{self.secondary}"
                + f",{self.tran_side.value}"
        )

    @classmethod
    def from_str(cls, _str: str) -> 'Analog':
        """从逗号分隔的字符串反序列化模拟量对象

        将配置文件中的模拟量通道字符串解析为Analog对象。
        该方法会自动处理通道名称中可能存在的非法逗号问题。

        参数:
            _str: 逗号分隔的模拟量通道字符串

        Returns:
            Analog: 解析后的模拟量通道对象

        异常:
            ValueError: 当字符串格式不正确或通道名称包含非法逗号时抛出
        """
        str_arr = _str.strip().split(',')
        if len(str_arr) > 13:
            str_arr = amend_channel_name_error(str_arr)
            logging.warning(f"{_str}参数超过规范的长度，怀疑ch_id(name)存在不合法的“,”,已尝试消除")
            if not str_arr:
                logging.error(f"{_str}参数存在不合法的“,”，尝试合并失败，请检查")
                raise ValueError(f"{_str}参数存在不合法的“,”，尝试合并失败，请检查")
        channel = super().from_str(','.join(str_arr[:4]))
        analog_dict = channel.model_dump()
        analog_dict['unit'] = Unit.from_value(str_arr[4], Unit.NONE)
        analog_dict['multiplier'] = parse_float(str_arr[5], 1.0)
        analog_dict['offset'] = parse_float(str_arr[6], 0.0)
        analog_dict['delay'] = parse_float(str_arr[7], 0.0)
        analog_dict['min_value'] = parse_float(str_arr[8], 0.0)
        analog_dict['max_value'] = parse_float(str_arr[9], 0.0)
        if len(str_arr) > 11:
            primary = parse_float(str_arr[10])
            secondary = parse_float(str_arr[11])
            analog_dict['primary'] = primary if primary != 0 else 1.0
            analog_dict['secondary'] = secondary if secondary != 0 else 1.0
        analog_dict['tran_side'] = TranSide.from_value(str_arr[12], TranSide.S) if len(str_arr) > 12 else TranSide.S
        return Analog(**analog_dict)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from comtrade_io.dmf.analog_channel import AnalogChannel
from comtrade_io.dmf.branch import ACCBranch, ACVBranch
from comtrade_io.dmf.bus import Bus
from comtrade_io.dmf.comtrade_model import ComtradeModel
from comtrade_io.dmf.dmf_channel import DmfChannel
from comtrade_io.dmf.line import Line
from comtrade_io.dmf.line_param import CG, MR, RX
from comtrade_io.dmf.status_channel import StatusChannel
from comtrade_io.dmf.transformer import Igap, Transformer, TransformerWinding, WG
